<?php

namespace Ardent\Undercurrent\Collector;

use RecursiveDirectoryIterator;
use RecursiveIteratorIterator;
use ReflectionClass;

class ClassAttributeCollector
{
    public function __construct(
        private readonly string  $path,
        private readonly ?string $attribute = null,
    )
    {
    }

    public function collect(): array
    {
        $classes = [];
        $files = new RecursiveIteratorIterator(new RecursiveDirectoryIterator($this->path));
        foreach ($files as $file) {
            if ($file->getExtension() !== 'php') {
                continue;
            }
            $class = $this->getClassFromFile($file);
            if (!$class) {
                continue;
            }
            $classes[] = $class;
        }
        return $classes;
    }

    private function getClassFromFile(\SplFileInfo $file): ?string
    {
        $contents = file_get_contents($file->getPathname());
        $tokens = token_get_all($contents);
        $namespace = '';
        $class = '';
        $classFound = false;
        $namespaceFound = false;
        foreach ($tokens as $token) {
            if (is_array($token)) {
                if ($token[0] === T_NAMESPACE) {
                    $namespaceFound = true;
                    continue;
                }
                if ($token[0] === T_CLASS) {
                    $classFound = true;
                    continue;
                }
                if ($namespaceFound && $token[0] === T_NAME_QUALIFIED) {
                    $namespace = $token[1];
                    $namespaceFound = false;
                    continue;
                }
                if ($classFound && $token[0] === T_STRING) {
                    $class .= $token[1];
                    break;
                }
            }
        }
        if (!$class) {
            return null;
        }
        $class = $namespace . "\\" . $class;
        if ($this->attribute && !in_array($this->attribute, $this->getClassAttributes($class))) {
            return null;
        }
        return $class;
    }

    private function getClassAttributes(string $class): array
    {
        $attributes = [];
        $reflection = new ReflectionClass($class);
        foreach ($reflection->getAttributes() as $attribute) {
            $attributes[] = $attribute->getName();
        }
        return $attributes;
    }
}