<?php

namespace Ardent\Undercurrent\Http;

use Ardent\Undercurrent\Container\ClassNotFoundException;
use Ardent\Undercurrent\Container\ContainerInterface;
use Ardent\Undercurrent\Logger\LoggerInterface;
use Exception;
use ReflectionMethod;

class GenericRouter implements RouterInterface
{
    public function __construct(
        private readonly ContainerInterface $container,
        private readonly LoggerInterface    $logger,
        private readonly RouterConfig       $config,
    )
    {
    }

    /**
     * @throws RouteNotFoundException
     */
    public function dispatch(RequestInterface $request): ResponseInterface
    {
        foreach ($this->config->getRoutes() as $route) {
            $params = $this->resolveParams($route->getRoute()->path, $request->getUri());

            if ($params === null) {
                continue;
            }

            $this->logger->add(sprintf('Matched route %s', $route->getRoute()->path));

            return $this->container->call(
                $route->getController(),
                $route->getMethod(),
                $params,
            );
        }

        throw new RouteNotFoundException($request);
    }

    private function resolveParams(string $routeUri, string $requestUri): ?array
    {
        if ($routeUri === $requestUri) {
            return [];
        }

        $result = preg_match_all('/{\w+}/', $routeUri, $tokens);

        if (!$result === null) {
            return null;
        }

        $tokens = $tokens[0];

        $matchingRegex = '/^' . str_replace(
                ['/', ...$tokens],
                ['\/', ...array_fill(0, count($tokens), '([\w\s]+)')],
                $routeUri
            ) . '$/';

        $result = preg_match($matchingRegex, $requestUri, $matches);

        if (!$result) {
            return null;
        }
        $matches = array_slice($matches, 1);

        return array_combine(array_map(fn($token) => trim($token, '{}'), $tokens), $matches);
    }
}