<?php

namespace Ardent\Undercurrent\Http;

use Ardent\Undercurrent\Attribute\Route;
use ReflectionAttribute;
use ReflectionClass;

class RouterConfig
{
    /**
     * @var array<RouteConfig>
     */
    private array $routes = [];

    public function addController(string $controller): self
    {
        $reflectionClass = new ReflectionClass($controller);

        foreach ($reflectionClass->getMethods() as $method) {
            $attributes = $method->getAttributes(Route::class, ReflectionAttribute::IS_INSTANCEOF);

            if (count($attributes) === 0) {
                continue;
            }

            $route = $attributes[0]->newInstance();

            $this->routes[] = new RouteConfig(
                $route,
                $controller,
                $method->getName(),
            );
        }

        return $this;
    }

    /**
     * @return array<RouteConfig>
     */
    public function getRoutes(): array
    {
        return $this->routes;
    }
}