<?php

namespace Ardent\Undercurrent\Kernel;

use Ardent\Undercurrent\Container\ContainerInterface;
use Ardent\Undercurrent\Container\GenericContainer;
use Ardent\Undercurrent\Http\GenericRequest;
use Ardent\Undercurrent\Http\GenericResponse;
use Ardent\Undercurrent\Http\GenericRouter;
use Ardent\Undercurrent\Http\MethodEnum;
use Ardent\Undercurrent\Http\RouterConfig;
use Ardent\Undercurrent\Http\RouterInterface;
use Ardent\Undercurrent\Http\StatusEnum;
use Ardent\Undercurrent\Logger\LogContainer;
use Ardent\Undercurrent\Logger\LoggerInterface;
use Exception;

class BaseKernel
{
    public function __invoke(): void
    {
        $container = (new GenericContainer());
        $container
            ->alias(RouterInterface::class, GenericRouter::class)
            ->alias(ContainerInterface::class, GenericContainer::class)
            ->alias(LoggerInterface::class, LogContainer::class)
            ->add(GenericContainer::class, fn($container) => $container)
            ->add(GenericRouter::class)
            ->add(LogContainer::class);

        $this->dependencies($container);

        $this->render($container);
    }

    private function render(GenericContainer $container): void
    {
        $request = new GenericRequest(
            MethodEnum::from($_SERVER['REQUEST_METHOD']),
            $_SERVER['REQUEST_URI'],
            $_REQUEST,
        );
        $router = $container->get(RouterInterface::class);
        $log = $container->get(LogContainer::class);
        try {
            $response = $router->dispatch($request);
        } catch (\Throwable $e) {
            $response = new GenericResponse(
                $e->getMessage(),
                StatusEnum::NOT_FOUND
            );
            //$response = $container->get(RouterConfig::class)->getExceptionRoute()->getController()::exception($e);
        }

        foreach ($log->getLogs() as $log) {
            echo sprintf('<p>%s</p>', $log);
        }

        http_response_code($response->getStatus()->value);

        foreach ($response->getHeaders() as $header) {
            header($header);
        }

        echo $response->getBody();
    }

    protected function addControllers(ContainerInterface $container, array $controllers): void
    {
        $config = new RouterConfig();

        foreach ($controllers as $controller) {
            $container->add($controller);
            $config->addController($controller);
        }

        $container->add(RouterConfig::class, fn() => $config);
    }

    protected function dependencies(ContainerInterface $container): void
    {
    }
}